@@ -1,26 +1,7 @@
 //! Bounds-checking for SPIR-V output.
 
 use super::{selection::Selection, Block, BlockContext, Error, IdGenerator, Instruction, Word};
-use crate::{arena::Handle, back::BoundsCheckPolicy};
-
-/// The results of emitting code for a left-hand-side expression.
-///
-/// On success, `write_expression_pointer` returns one of these.
-pub(super) enum ExpressionPointer {
-    /// The pointer to the expression's value is available, as the value of the
-    /// expression with the given id.
-    Ready { pointer_id: Word },
-
-    /// The access expression must be conditional on the value of `condition`, a boolean
-    /// expression that is true if all indices are in bounds. If `condition` is true, then
-    /// `access` is an `OpAccessChain` instruction that will compute a pointer to the
-    /// expression's value. If `condition` is false, then executing `access` would be
-    /// undefined behavior.
-    Conditional {
-        condition: Word,
-        access: Instruction,
-    },
-}
+use crate::{arena::Handle, proc::BoundsCheckPolicy};
 
 /// The results of performing a bounds check.
 ///
@@ -62,13 +43,10 @@ impl<'w> BlockContext<'w> {
         let (structure_id, last_member_index) = match self.ir_function.expressions[array] {
             crate::Expression::AccessIndex { base, index } => {
                 match self.ir_function.expressions[base] {
-                    crate::Expression::GlobalVariable(handle) => {
-                        (self.writer.global_variables[handle.index()].id, index)
-                    }
-                    crate::Expression::FunctionArgument(index) => {
-                        let parameter_id = self.function.parameter_id(index);
-                        (parameter_id, index)
-                    }
+                    crate::Expression::GlobalVariable(handle) => (
+                        self.writer.global_variables[handle.index()].access_id,
+                        index,
+                    ),
                     _ => return Err(Error::Validation("array length expression")),
                 }
             }
@@ -109,10 +87,6 @@ impl<'w> BlockContext<'w> {
                 let length_id = self.write_runtime_array_length(sequence, block)?;
                 Ok(MaybeKnown::Computed(length_id))
             }
-            crate::proc::IndexableLength::Specializable(constant) => {
-                let length_id = self.writer.constant_ids[constant.index()];
-                Ok(MaybeKnown::Computed(length_id))
-            }
         }
     }
 
@@ -346,22 +320,11 @@ impl<'w> BlockContext<'w> {
         index: Handle<crate::Expression>,
         block: &mut Block,
     ) -> Result<BoundsCheckResult, Error> {
-        // Should this access be covered by `index_bounds_check_policy` or
-        // `buffer_bounds_check_policy`?
-        let is_buffer = match *self.fun_info[base].ty.inner_with(&self.ir_module.types) {
-            crate::TypeInner::Pointer { class, .. }
-            | crate::TypeInner::ValuePointer { class, .. } => match class {
-                crate::StorageClass::Storage { access: _ } | crate::StorageClass::Uniform => true,
-                _ => false,
-            },
-            _ => false,
-        };
-
-        let policy = if is_buffer {
-            self.writer.bounds_check_policies.buffer
-        } else {
-            self.writer.bounds_check_policies.index
-        };
+        let policy = self.writer.bounds_check_policies.choose_policy(
+            base,
+            &self.ir_module.types,
+            self.fun_info,
+        );
 
         Ok(match policy {
             BoundsCheckPolicy::Restrict => self.write_restricted_index(base, index, block)?,
