@@ -418,6 +418,69 @@ impl Context {
         Ok((expr, meta))
     }
 
+    fn lower_store(
+        &mut self,
+        pointer: Handle<Expression>,
+        value: Handle<Expression>,
+        meta: Span,
+        body: &mut Block,
+    ) {
+        if let Expression::Swizzle {
+            size,
+            mut vector,
+            pattern,
+        } = self.expressions[pointer]
+        {
+            // Stores to swizzled values are not directly supported,
+            // lower them as series of per-component stores.
+            let size = match size {
+                VectorSize::Bi => 2,
+                VectorSize::Tri => 3,
+                VectorSize::Quad => 4,
+            };
+
+            if let Expression::Load { pointer } = self.expressions[vector] {
+                vector = pointer;
+            }
+
+            #[allow(clippy::needless_range_loop)]
+            for index in 0..size {
+                let dst = self.add_expression(
+                    Expression::AccessIndex {
+                        base: vector,
+                        index: pattern[index].index(),
+                    },
+                    meta,
+                    body,
+                );
+                let src = self.add_expression(
+                    Expression::AccessIndex {
+                        base: value,
+                        index: index as u32,
+                    },
+                    meta,
+                    body,
+                );
+
+                self.emit_flush(body);
+                self.emit_start();
+
+                body.push(
+                    Statement::Store {
+                        pointer: dst,
+                        value: src,
+                    },
+                    meta,
+                );
+            }
+        } else {
+            self.emit_flush(body);
+            self.emit_start();
+
+            body.push(Statement::Store { pointer, value }, meta);
+        }
+    }
+
     /// Internal implementation of [`lower`](Self::lower)
     fn lower_inner(
         &mut self,
@@ -492,10 +555,10 @@ impl Context {
 
                 parser.field_selection(self, ExprPos::Lhs == pos, body, base, field, meta)?
             }
-            HirExprKind::Constant(constant) if pos == ExprPos::Rhs => {
+            HirExprKind::Constant(constant) if pos != ExprPos::Lhs => {
                 self.add_expression(Expression::Constant(constant), meta, body)
             }
-            HirExprKind::Binary { left, op, right } if pos == ExprPos::Rhs => {
+            HirExprKind::Binary { left, op, right } if pos != ExprPos::Lhs => {
                 let (mut left, left_meta) =
                     self.lower_expect_inner(stmt, parser, left, pos, body)?;
                 let (mut right, right_meta) =
@@ -599,7 +662,7 @@ impl Context {
                     _ => self.add_expression(Expression::Binary { left, op, right }, meta, body),
                 }
             }
-            HirExprKind::Unary { op, expr } if pos == ExprPos::Rhs => {
+            HirExprKind::Unary { op, expr } if pos != ExprPos::Lhs => {
                 let expr = self.lower_expect_inner(stmt, parser, expr, pos, body)?.0;
 
                 self.add_expression(Expression::Unary { op, expr }, meta, body)
@@ -694,68 +757,18 @@ impl Context {
                     self.implicit_conversion(parser, &mut value, value_meta, kind, width)?;
                 }
 
-                if let Expression::Swizzle {
-                    size,
-                    mut vector,
-                    pattern,
-                } = self.expressions[pointer]
-                {
-                    // Stores to swizzled values are not directly supported,
-                    // lower them as series of per-component stores.
-                    let size = match size {
-                        VectorSize::Bi => 2,
-                        VectorSize::Tri => 3,
-                        VectorSize::Quad => 4,
-                    };
-
-                    if let Expression::Load { pointer } = self.expressions[vector] {
-                        vector = pointer;
-                    }
-
-                    #[allow(clippy::needless_range_loop)]
-                    for index in 0..size {
-                        let dst = self.add_expression(
-                            Expression::AccessIndex {
-                                base: vector,
-                                index: pattern[index].index(),
-                            },
-                            meta,
-                            body,
-                        );
-                        let src = self.add_expression(
-                            Expression::AccessIndex {
-                                base: value,
-                                index: index as u32,
-                            },
-                            meta,
-                            body,
-                        );
-
-                        self.emit_flush(body);
-                        self.emit_start();
-
-                        body.push(
-                            Statement::Store {
-                                pointer: dst,
-                                value: src,
-                            },
-                            meta,
-                        );
-                    }
-                } else {
-                    self.emit_flush(body);
-                    self.emit_start();
-
-                    body.push(Statement::Store { pointer, value }, meta);
-                }
+                self.lower_store(pointer, value, meta, body);
 
                 value
             }
             HirExprKind::PrePostfix { op, postfix, expr } if ExprPos::Lhs != pos => {
-                let pointer = self
-                    .lower_expect_inner(stmt, parser, expr, ExprPos::Lhs, body)?
-                    .0;
-                let left = self.add_expression(Expression::Load { pointer }, meta, body);
+                let (pointer, _) =
+                    self.lower_expect_inner(stmt, parser, expr, ExprPos::Lhs, body)?;
+                let left = if let Expression::Swizzle { .. } = self.expressions[pointer] {
+                    pointer
+                } else {
+                    self.add_expression(Expression::Load { pointer }, meta, body)
+                };
 
                 let make_constant_inner = |kind, width| {
                     let value = match kind {
@@ -844,10 +857,7 @@ impl Context {
 
                 let value = self.add_expression(Expression::Binary { op, left, right }, meta, body);
 
-                self.emit_flush(body);
-                self.emit_start();
-
-                body.push(Statement::Store { pointer, value }, meta);
+                self.lower_store(pointer, value, meta, body);
 
                 if postfix {
                     left
