@@ -1,9 +1,9 @@
 use super::{
     ast::Precision,
-    token::{Directive, DirectiveKind, SourceMetadata, Token, TokenValue},
+    token::{Directive, DirectiveKind, Token, TokenValue},
     types::parse_type,
 };
-use crate::{FastHashMap, StorageAccess};
+use crate::{FastHashMap, Span, StorageAccess};
 use pp_rs::{
     pp::Preprocessor,
     token::{PreprocessorError, Punct, TokenValue as PPTokenValue},
@@ -13,7 +13,7 @@ use pp_rs::{
 #[cfg_attr(test, derive(PartialEq))]
 pub struct LexerResult {
     pub kind: LexerResultKind,
-    pub meta: SourceMetadata,
+    pub meta: Span,
 }
 
 #[derive(Debug)]
@@ -71,6 +71,7 @@ impl<'a> Iterator for Lexer<'a> {
                     "out" => TokenValue::Out,
                     "uniform" => TokenValue::Uniform,
                     "buffer" => TokenValue::Buffer,
+                    "shared" => TokenValue::Shared,
                     "flat" => TokenValue::Interpolation(crate::Interpolation::Flat),
                     "noperspective" => TokenValue::Interpolation(crate::Interpolation::Linear),
                     "smooth" => TokenValue::Interpolation(crate::Interpolation::Perspective),
@@ -201,9 +202,10 @@ mod tests {
     use pp_rs::token::{Integer, Location, Token as PPToken, TokenValue as PPTokenValue};
 
     use super::{
-        super::token::{Directive, DirectiveKind, SourceMetadata, Token, TokenValue},
+        super::token::{Directive, DirectiveKind, Token, TokenValue},
         Lexer, LexerResult, LexerResultKind,
     };
+    use crate::Span;
 
     #[test]
     fn lex_tokens() {
@@ -230,7 +232,7 @@ mod tests {
                         location
                     }]
                 }),
-                meta: SourceMetadata { start: 1, end: 8 }
+                meta: Span::new(1, 8)
             }
         );
         assert_eq!(
@@ -238,9 +240,9 @@ mod tests {
             LexerResult {
                 kind: LexerResultKind::Token(Token {
                     value: TokenValue::Void,
-                    meta: SourceMetadata { start: 13, end: 17 }
+                    meta: Span::new(13, 17)
                 }),
-                meta: SourceMetadata { start: 13, end: 17 }
+                meta: Span::new(13, 17)
             }
         );
         assert_eq!(
@@ -248,9 +250,9 @@ mod tests {
             LexerResult {
                 kind: LexerResultKind::Token(Token {
                     value: TokenValue::Identifier("main".into()),
-                    meta: SourceMetadata { start: 18, end: 22 }
+                    meta: Span::new(18, 22)
                 }),
-                meta: SourceMetadata { start: 18, end: 22 }
+                meta: Span::new(18, 22)
             }
         );
         assert_eq!(
@@ -258,9 +260,9 @@ mod tests {
             LexerResult {
                 kind: LexerResultKind::Token(Token {
                     value: TokenValue::LeftParen,
-                    meta: SourceMetadata { start: 23, end: 24 }
+                    meta: Span::new(23, 24)
                 }),
-                meta: SourceMetadata { start: 23, end: 24 }
+                meta: Span::new(23, 24)
             }
         );
         assert_eq!(
@@ -268,9 +270,9 @@ mod tests {
             LexerResult {
                 kind: LexerResultKind::Token(Token {
                     value: TokenValue::RightParen,
-                    meta: SourceMetadata { start: 24, end: 25 }
+                    meta: Span::new(24, 25)
                 }),
-                meta: SourceMetadata { start: 24, end: 25 }
+                meta: Span::new(24, 25)
             }
         );
         assert_eq!(
@@ -278,9 +280,9 @@ mod tests {
             LexerResult {
                 kind: LexerResultKind::Token(Token {
                     value: TokenValue::LeftBrace,
-                    meta: SourceMetadata { start: 26, end: 27 }
+                    meta: Span::new(26, 27)
                 }),
-                meta: SourceMetadata { start: 26, end: 27 }
+                meta: Span::new(26, 27)
             }
         );
         assert_eq!(
@@ -288,9 +290,9 @@ mod tests {
             LexerResult {
                 kind: LexerResultKind::Token(Token {
                     value: TokenValue::RightBrace,
-                    meta: SourceMetadata { start: 27, end: 28 }
+                    meta: Span::new(27, 28)
                 }),
-                meta: SourceMetadata { start: 27, end: 28 }
+                meta: Span::new(27, 28)
             }
         );
         assert_eq!(lex.next(), None);
