@@ -1,5 +1,7 @@
 //! Parsers which load shaders into memory.
 
+mod interpolator;
+
 #[cfg(feature = "glsl-in")]
 pub mod glsl;
 #[cfg(feature = "spv-in")]
@@ -8,7 +10,7 @@ pub mod spv;
 pub mod wgsl;
 
 use crate::{
-    arena::{Arena, Handle},
+    arena::{Arena, Handle, UniqueArena},
     proc::{ResolveContext, ResolveError, TypeResolution},
 };
 use std::ops;
@@ -36,7 +38,7 @@ impl Emitter {
         let start_len = self.start_len.take().unwrap();
         if start_len != arena.len() {
             #[allow(unused_mut)]
-            let mut span = crate::span::Span::Unknown;
+            let mut span = crate::span::Span::default();
             let range = arena.range_from(start_len);
             #[cfg(feature = "span")]
             for handle in range.clone() {
@@ -79,7 +81,7 @@ impl Typifier {
     pub fn get<'a>(
         &'a self,
         expr_handle: Handle<crate::Expression>,
-        types: &'a Arena<crate::Type>,
+        types: &'a UniqueArena<crate::Type>,
     ) -> &'a crate::TypeInner {
         self.resolutions[expr_handle.index()].inner_with(types)
     }
@@ -127,3 +129,20 @@ impl ops::Index<Handle<crate::Expression>> for Typifier {
         &self.resolutions[handle.index()]
     }
 }
+
+/// Helper function used for aligning `value` to the next multiple of `align`
+///
+/// # Notes:
+/// - `align` must be a power of two.
+/// - The returned value will be greater or equal to `value`.
+/// # Examples:
+/// ```ignore
+/// assert_eq!(0, align_up(0, 16));
+/// assert_eq!(16, align_up(1, 16));
+/// assert_eq!(16, align_up(16, 16));
+/// assert_eq!(334, align_up(333, 2));
+/// assert_eq!(384, align_up(257, 128));
+/// ```
+pub fn align_up(value: u32, align: u32) -> u32 {
+    ((value.wrapping_sub(1)) & !(align - 1)).wrapping_add(align)
+}
