@@ -209,18 +209,14 @@ mod arena;
 pub mod back;
 mod block;
 pub mod front;
+pub mod keywords;
 pub mod proc;
 mod span;
 pub mod valid;
 
 pub use crate::arena::{Arena, Handle, Range, UniqueArena};
 
-use std::{
-    collections::{HashMap, HashSet},
-    hash::BuildHasherDefault,
-};
-
-pub use crate::span::Span;
+pub use crate::span::{Span, SpanContext, WithSpan};
 #[cfg(feature = "deserialize")]
 use serde::Deserialize;
 #[cfg(feature = "serialize")]
@@ -230,9 +226,9 @@ use serde::Serialize;
 pub const BOOL_WIDTH: Bytes = 1;
 
 /// Hash map that is faster but not resilient to DoS attacks.
-pub type FastHashMap<K, T> = HashMap<K, T, BuildHasherDefault<fxhash::FxHasher>>;
+pub type FastHashMap<K, T> = rustc_hash::FxHashMap<K, T>;
 /// Hash set that is faster but not resilient to DoS attacks.
-pub type FastHashSet<K> = HashSet<K, BuildHasherDefault<fxhash::FxHasher>>;
+pub type FastHashSet<K> = rustc_hash::FxHashSet<K>;
 
 /// Map of expressions that have associated variable names
 pub(crate) type NamedExpressions = FastHashMap<Handle<Expression>, String>;
@@ -631,11 +627,10 @@ pub enum TypeInner {
     /// An `Array` is [`SIZED`] unless its `size` is [`Dynamic`].
     /// Dynamically-sized arrays may only appear in a few situations:
     ///
-    /// -   They may appear as the last member of a [`Struct`] whose `top_level`
-    ///     flag is set.
+    /// -   They may appear as the last member of a [`Struct`].
     ///
     /// -   They may appear as the base type of a [`Pointer`]. An
-    ///     [`AccessIndex`] expression referring to a top-level struct's final
+    ///     [`AccessIndex`] expression referring to a struct's final
     ///     unsized array member would have such a pointer type. However, such
     ///     pointer types may only appear as the types of such intermediate
     ///     expressions. They are not [`DATA`], and cannot be stored in
@@ -659,20 +654,13 @@ pub enum TypeInner {
     /// `DATA` as well.
     ///
     /// Member types must be [`SIZED`], except for the final member of a
-    /// top-level struct, which may be a dynamically sized [`Array`]. The
+    /// struct, which may be a dynamically sized [`Array`]. The
     /// `Struct` type itself is `SIZED` when all its members are `SIZED`.
     ///
-    /// When `top_level` is true, this `Struct` represents the contents of a
-    /// buffer resource occupying a single binding slot in a shader's resource
-    /// interface. Top-level `Struct`s may not be used as members of any other
-    /// struct, or as array elements.
-    ///
     /// [`DATA`]: crate::valid::TypeFlags::DATA
     /// [`SIZED`]: crate::valid::TypeFlags::SIZED
     /// [`Array`]: TypeInner::Array
     Struct {
-        /// This struct serves as the type of a binding slot in a shader's resource interface.
-        top_level: bool,
         members: Vec<StructMember>,
         //TODO: should this be unaligned?
         span: u32,
@@ -987,7 +975,7 @@ pub enum ImageQuery {
 
 /// Component selection for a vector swizzle.
 #[repr(u8)]
-#[derive(Clone, Copy, Debug, PartialEq)]
+#[derive(Clone, Copy, Debug, PartialEq, PartialOrd)]
 #[cfg_attr(feature = "serialize", derive(Serialize))]
 #[cfg_attr(feature = "deserialize", derive(Deserialize))]
 pub enum SwizzleComponent {
@@ -1140,6 +1128,9 @@ pub enum Expression {
     ImageSample {
         image: Handle<Expression>,
         sampler: Handle<Expression>,
+        /// If Some(), this operation is a gather operation
+        /// on the selected component.
+        gather: Option<SwizzleComponent>,
         coordinate: Handle<Expression>,
         array_index: Option<Handle<Expression>>,
         offset: Option<Handle<Constant>>,
@@ -1301,6 +1292,16 @@ pub enum Expression {
 
 pub use block::Block;
 
+/// The value of the switch case
+// Clone is used only for error reporting and is not intended for end users
+#[derive(Clone, Debug)]
+#[cfg_attr(feature = "serialize", derive(Serialize))]
+#[cfg_attr(feature = "deserialize", derive(Deserialize))]
+pub enum SwitchValue {
+    Integer(i32),
+    Default,
+}
+
 /// A case for a switch statement.
 // Clone is used only for error reporting and is not intended for end users
 #[derive(Clone, Debug)]
@@ -1308,7 +1309,7 @@ pub use block::Block;
 #[cfg_attr(feature = "deserialize", derive(Deserialize))]
 pub struct SwitchCase {
     /// Value, upon which the case is considered true.
-    pub value: i32,
+    pub value: SwitchValue,
     /// Body of the case.
     pub body: Block,
     /// If true, the control flow continues to the next case in the list,
@@ -1341,7 +1342,6 @@ pub enum Statement {
     Switch {
         selector: Handle<Expression>, //int
         cases: Vec<SwitchCase>,
-        default: Block,
     },
 
     /// Executes a block repeatedly.
