@@ -22,7 +22,7 @@ pub enum NameKey {
 /// that may need identifiers in a textual backend.
 #[derive(Default)]
 pub struct Namer {
-    /// The last numeric suffix used for each base name.
+    /// The last numeric suffix used for each base name. Zero means "no suffix".
     unique: FastHashMap<String, u32>,
     keywords: FastHashSet<String>,
     reserved_prefixes: Vec<String>,
@@ -64,7 +64,7 @@ impl Namer {
 
         for prefix in &self.reserved_prefixes {
             if base.starts_with(prefix) {
-                return Cow::Owned(format!("gen_{}", base));
+                return format!("gen_{}", base).into();
             }
         }
 
@@ -79,7 +79,8 @@ impl Namer {
     /// - is different from any identifier previously constructed by this
     ///   `Namer`.
     ///
-    /// Guarantee uniqueness by applying a numeric suffix when necessary.
+    /// Guarantee uniqueness by applying a numeric suffix when necessary. If `label_raw`
+    /// itself ends with digits, separate them from the suffix with an underscore.
     pub fn call(&mut self, label_raw: &str) -> String {
         use std::fmt::Write as _; // for write!-ing to Strings
 
