@@ -7,6 +7,7 @@ Figures out the following properties:
 !*/
 
 use super::{CallError, ExpressionError, FunctionError, ModuleInfo, ShaderStages, ValidationFlags};
+use crate::span::{AddSpan as _, WithSpan};
 use crate::{
     arena::{Arena, Handle},
     proc::{ResolveContext, TypeResolution},
@@ -307,7 +308,7 @@ impl FunctionInfo {
         info: &Self,
         arguments: &[Handle<crate::Expression>],
         expression_arena: &Arena<crate::Expression>,
-    ) -> Result<FunctionUniformity, FunctionError> {
+    ) -> Result<FunctionUniformity, WithSpan<FunctionError>> {
         for key in info.sampling_set.iter() {
             self.sampling_set.insert(key.clone());
         }
@@ -318,7 +319,10 @@ impl FunctionInfo {
                     let handle = arguments[i as usize];
                     expression_arena[handle]
                         .to_global_or_argument()
-                        .map_err(|error| FunctionError::Expression { handle, error })?
+                        .map_err(|error| {
+                            FunctionError::Expression { handle, error }
+                                .with_span_handle(handle, expression_arena)
+                        })?
                 }
             };
 
@@ -328,7 +332,10 @@ impl FunctionInfo {
                     let handle = arguments[i as usize];
                     expression_arena[handle]
                         .to_global_or_argument()
-                        .map_err(|error| FunctionError::Expression { handle, error })?
+                        .map_err(|error| {
+                            FunctionError::Expression { handle, error }
+                                .with_span_handle(handle, expression_arena)
+                        })?
                 }
             };
 
@@ -455,6 +462,7 @@ impl FunctionInfo {
             E::ImageSample {
                 image,
                 sampler,
+                gather: _,
                 coordinate,
                 array_index,
                 offset: _,
@@ -608,15 +616,15 @@ impl FunctionInfo {
     #[allow(clippy::or_fun_call)]
     fn process_block(
         &mut self,
-        statements: &[crate::Statement],
+        statements: &crate::Block,
         other_functions: &[FunctionInfo],
         mut disruptor: Option<UniformityDisruptor>,
         expression_arena: &Arena<crate::Expression>,
-    ) -> Result<FunctionUniformity, FunctionError> {
+    ) -> Result<FunctionUniformity, WithSpan<FunctionError>> {
         use crate::Statement as S;
 
         let mut combined_uniformity = FunctionUniformity::new();
-        for statement in statements {
+        for (statement, &span) in statements.span_iter() {
             let uniformity = match *statement {
                 S::Emit(ref range) => {
                     let mut requirements = UniformityRequirements::empty();
@@ -629,7 +637,8 @@ impl FunctionInfo {
                             && !req.is_empty()
                         {
                             if let Some(cause) = disruptor {
-                                return Err(FunctionError::NonUniformControlFlow(req, expr, cause));
+                                return Err(FunctionError::NonUniformControlFlow(req, expr, cause)
+                                    .with_span_handle(expr, expression_arena));
                             }
                         }
                         requirements |= req;
@@ -686,7 +695,6 @@ impl FunctionInfo {
                 S::Switch {
                     selector,
                     ref cases,
-                    ref default,
                 } => {
                     let selector_nur = self.add_ref(selector);
                     let branch_disruptor =
@@ -707,14 +715,7 @@ impl FunctionInfo {
                         };
                         uniformity = uniformity | case_uniformity;
                     }
-                    // using the disruptor inherited from the last fall-through chain
-                    let default_exit = self.process_block(
-                        default,
-                        other_functions,
-                        case_disruptor,
-                        expression_arena,
-                    )?;
-                    uniformity | default_exit
+                    uniformity
                 }
                 S::Loop {
                     ref body,
@@ -776,7 +777,8 @@ impl FunctionInfo {
                         FunctionError::InvalidCall {
                             function,
                             error: CallError::ForwardDeclaredFunction,
-                        },
+                        }
+                        .with_span_static(span, "forward call"),
                     )?;
                     //Note: the result is validated by the Validator, not here
                     self.process_call(info, arguments, expression_arena)?
@@ -811,7 +813,7 @@ impl ModuleInfo {
         fun: &crate::Function,
         module: &crate::Module,
         flags: ValidationFlags,
-    ) -> Result<FunctionInfo, FunctionError> {
+    ) -> Result<FunctionInfo, WithSpan<FunctionError>> {
         let mut info = FunctionInfo {
             flags,
             available_stages: ShaderStages::all(),
@@ -839,7 +841,8 @@ impl ModuleInfo {
                 &self.functions,
                 &resolve_context,
             ) {
-                return Err(FunctionError::Expression { handle, error });
+                return Err(FunctionError::Expression { handle, error }
+                    .with_span_handle(handle, &fun.expressions));
             }
         }
 
@@ -979,7 +982,12 @@ fn uniform_control_flow() {
         .into(),
     };
     assert_eq!(
-        info.process_block(&[stmt_emit1, stmt_if_uniform], &[], None, &expressions),
+        info.process_block(
+            &vec![stmt_emit1, stmt_if_uniform].into(),
+            &[],
+            None,
+            &expressions
+        ),
         Ok(FunctionUniformity {
             result: Uniformity {
                 non_uniform_result: None,
@@ -1005,12 +1013,18 @@ fn uniform_control_flow() {
         reject: crate::Block::new(),
     };
     assert_eq!(
-        info.process_block(&[stmt_emit2, stmt_if_non_uniform], &[], None, &expressions),
+        info.process_block(
+            &vec![stmt_emit2, stmt_if_non_uniform].into(),
+            &[],
+            None,
+            &expressions
+        ),
         Err(FunctionError::NonUniformControlFlow(
             UniformityRequirements::DERIVATIVE,
             derivative_expr,
             UniformityDisruptor::Expression(non_uniform_global_expr)
-        )),
+        )
+        .with_span()),
     );
     assert_eq!(info[derivative_expr].ref_count, 1);
     assert_eq!(info[non_uniform_global], GlobalUse::READ);
@@ -1021,7 +1035,7 @@ fn uniform_control_flow() {
     };
     assert_eq!(
         info.process_block(
-            &[stmt_emit3, stmt_return_non_uniform],
+            &vec![stmt_emit3, stmt_return_non_uniform].into(),
             &[],
             Some(UniformityDisruptor::Return),
             &expressions
@@ -1048,7 +1062,7 @@ fn uniform_control_flow() {
     let stmt_kill = S::Kill;
     assert_eq!(
         info.process_block(
-            &[stmt_emit4, stmt_assign, stmt_kill, stmt_return_pointer],
+            &vec![stmt_emit4, stmt_assign, stmt_kill, stmt_return_pointer].into(),
             &[],
             Some(UniformityDisruptor::Discard),
             &expressions
